@@ -18,6 +18,7 @@ along with OpenSesame.  If not, see <http://www.gnu.org/licenses/>.
 """
 
 from libopensesame.py3compat import *
+import os
 import sys
 from libopensesame.oslogging import oslogger
 from libqtopensesame.misc.config import cfg
@@ -32,6 +33,8 @@ from jupyter_tabwidget.constants import (
     DEFAULT_RUN_FILE_CMD,
     TRANSPARENT_KERNELS
 )
+from libqtopensesame.misc.translate import translation_context
+_ = translation_context(u'JupyterConsole', category=u'extension')
 
 
 class JupyterConsole(BaseWidget):
@@ -183,6 +186,10 @@ class JupyterConsole(BaseWidget):
         if self._kernel in TRANSPARENT_KERNELS:
             return self._jupyter_widget.get_workspace_variable(name)
         return None
+    
+    def running(self):
+        
+        return self._jupyter_widget.running()
 
     def set_workspace_globals(self, global_dict):
 
@@ -194,7 +201,7 @@ class JupyterConsole(BaseWidget):
             CHANGE_DIR_CMD.get(
                 self._kernel,
                 DEFAULT_CHANGE_DIR_CMD
-            ).format(path)
+            ).format(path=path)
         )
 
     def run_file(self, path):
@@ -203,9 +210,38 @@ class JupyterConsole(BaseWidget):
             RUN_FILE_CMD.get(
                 self._kernel,
                 DEFAULT_RUN_FILE_CMD
-            ).format(path)
+            ).format(path=path)
         )
 
+    def run_debug(self, path, breakpoints):
+
+        # This should be changed into a kernel-agnostic implementation
+        if (
+            self._kernel not in ('python', 'python2', 'python3') or
+            self._inprocess
+        ):
+            self.extension_manager.fire(
+                'notify',
+                message=_(u'The {} (inprocess={}) kernel does not support debugging').format(
+                    self._kernel,
+                    self._inprocess
+                )
+            )
+            return
+        code = self.extension_manager.provide(
+            'python_debugger_code',
+            path=path,
+            breakpoints=breakpoints
+        )
+        if not code:
+            oslogger.warning('no Python debugger code received')
+            return
+        silent_code, nonsilent_code = code
+        # RapunzelPDB and debugfile() are silently loaded into the workspace,
+        # and then the debugger is callend in a nonsilent way.`
+        self._jupyter_widget._kernel_client.execute(silent_code, silent=True)
+        self.execute(nonsilent_code)
+
     def check_syntax(self, code):
 
         if 'python' not in self._kernel:
