@@ -31,6 +31,7 @@ from qtpy.QtWidgets import QFileDialog, QMessageBox, QPushButton
 from opensesame_ide import FolderBrowserDockWidget, MenuBar
 from libqtopensesame.misc.translate import translation_context
 from pyqode.core import widgets
+from pyqode.core.api.utils import TextHelper
 from pyqode_extras.widgets import FallbackCodeEdit
 _ = translation_context(u'OpenSesameIDE', category=u'extension')
 
@@ -52,6 +53,7 @@ class OpenSesameIDE(BaseExtension):
         self._scetw.fallback_editor = FallbackCodeEdit
         self._scetw.tab_name = u'OpenSesameIDE'
         self._scetw.main_tab_widget.tab_closed.connect(self._on_editor_close)
+        self._scetw.tab_bar_visible = cfg.opensesame_ide_show_tab_bar
         self._add_ide_tab()
         self._dock_widgets = {}
         self._set_ignore_patterns()
@@ -61,6 +63,13 @@ class OpenSesameIDE(BaseExtension):
         self.main_window.setWindowIcon(
             self.theme.qicon(u'rapunzel')
         )
+        
+    @property
+    def dock_widgets(self):
+        
+        # Turn it into a list so that we don't get race conditions when the
+        # dict changes during an iteration
+        return list(self._dock_widgets.values())
 
     def event_setting_changed(self, setting, value):
         
@@ -100,6 +109,15 @@ class OpenSesameIDE(BaseExtension):
             return u''
         return editor.toPlainText()
         
+    def provide_ide_current_word(self):
+        
+        editor = self._current_editor()
+        if editor is None:
+            return u''
+        return TextHelper(editor).word_under_cursor(
+            select_whole_word=True
+        ).selectedText()
+        
     def toggle_breakpoint(self):
 
         editor = self._current_editor()
@@ -299,7 +317,7 @@ class OpenSesameIDE(BaseExtension):
 
         return all(
             dockwidget.isVisible()
-            for dockwidget in self._dock_widgets.values()
+            for dockwidget in self.dock_widgets
         )
 
     def toggle_folder_browsers(self):
@@ -308,7 +326,7 @@ class OpenSesameIDE(BaseExtension):
         oslogger.debug(
             'setting folder-browser visibility to {}'.format(hidden)
         )
-        for dockwidget in self._dock_widgets.values():
+        for dockwidget in self.dock_widgets:
             dockwidget.setVisible(hidden)
 
     def _select_logical_line(self, editor, scan_width=8):
@@ -478,7 +496,7 @@ class OpenSesameIDE(BaseExtension):
     def quick_select_files(self):
 
         haystack = []
-        for dock_widget in self._dock_widgets.values():
+        for dock_widget in self.dock_widgets:
             strip_first = len(os.path.split(dock_widget.path)[0])
             for path in dock_widget.file_list:
                 label = path[strip_first + 1:]
@@ -507,7 +525,7 @@ class OpenSesameIDE(BaseExtension):
 
     def project_files(self, extra_ignore_pattern=None):
 
-        for dock_widget in self._dock_widgets.values():
+        for dock_widget in self.dock_widgets:
             for path in dock_widget.file_list:
                 yield path
                 
@@ -569,7 +587,7 @@ class OpenSesameIDE(BaseExtension):
     def _open_folder(self, path):
 
         path = os.path.abspath(path)
-        if path in self._dock_widgets:
+        if path in list(self._dock_widgets):
             return
         oslogger.debug(u'adding folder browser: {}'.format(path))
         dock_widget = FolderBrowserDockWidget(self.main_window, self, path)
@@ -585,17 +603,17 @@ class OpenSesameIDE(BaseExtension):
 
     def close_all_folders(self):
 
-        for dockwidget in list(self._dock_widgets.values()):
+        for dockwidget in self.dock_widgets:
             dockwidget.close()
 
     def locate_file_in_folder(self):
 
-        for dockwidget in self._dock_widgets.values():
+        for dockwidget in self.dock_widgets:
             dockwidget.setVisible(True)
         editor = self._current_editor()
         if editor is None:
             return
-        for dock_widget in self._dock_widgets.values():
+        for dock_widget in self.dock_widgets:
             dock_widget.select_path(editor.file.path)
         if not self.folder_browsers_visible():
             self.toggle_folder_browsers()
@@ -713,7 +731,7 @@ class OpenSesameIDE(BaseExtension):
             self.main_window.removeToolBar(self.main_window.ui.toolbar_main)
             # Always start with the folder browsers visible. This avoids
             # inconsistencies in their visibility.
-            for dockwidget in self._dock_widgets.values():
+            for dockwidget in self.dock_widgets:
                 dockwidget.setVisible(True)
 
         return inner
@@ -734,7 +752,7 @@ class OpenSesameIDE(BaseExtension):
 
     def _remember_open_folders(self):
 
-        folders = [d.path for d in self._dock_widgets.values()]
+        folders = [d.path for d in self.dock_widgets]
         cfg.opensesame_ide_last_folder = u';'.join(folders)
 
     def _restore_open_folders(self):
@@ -748,21 +766,31 @@ class OpenSesameIDE(BaseExtension):
             folders = [os.getcwd()]
         for folder in folders:
             self._open_folder(folder)
+    
+    @BaseExtension.as_thread(wait=750)
+    def _open_from_command_line(self, path):
+
+        """This function is run with a timer so that the other extensions have
+        a chance to initialize first.
+        """
+
+        if os.path.isfile(path):
+            self.open_document(path)
+        elif os.path.isdir(path):
+            self._open_folder(path)
 
     def _parse_command_line(self):
 
         for arg in sys.argv[1:]:
-            if os.path.isfile(arg):
-                self.open_document(arg)
-            elif os.path.isdir(arg):
-                self._open_folder(arg)
+            if os.path.exists(arg):
+                self._open_from_command_line(arg)
 
     def _current_project_file(self):
 
         editor = self._current_editor()
         if editor is None:
             return None
-        for d in self._dock_widgets.values():
+        for d in self.dock_widgets:
             if not editor.file.path.startswith(d.path):
                 continue
             project_file_path = os.path.join(
